; ===================================================================
; Project: GUIRegisterLib Library
; Description: An extension of GUIRegisterMsg() in which a single WM_MESSAGE can be registered with multiple AutoIt windows.
; Author: guinness
; ===================================================================
#include-once

#Region Members Exported
#cs
	_GUIRegisterMsgEx() - Register/Unregister a single WM_MESSAGE with multiple AutoIt windows.
#ce
#EndRegion Members Exported

#Region Includes
#include <GUIConstants.au3>
#include <StringConstants.au3>
#include <WindowsConstants.au3>
#EndRegion Includes

#Region Global Variables
Global Const $GUIREGISTERMSGEX_GUID = 'C9028362-1DC8-11E4-9A4A-CBE0E2E253B6'
Global Enum $GUIREGISTERMSGEX_ADD = 1025, $GUIREGISTERMSGEX_DELETE ; 1025 = $WM_USER + 1
Global Const $GUIREGISTERMSGEX = 0
Global Enum $GUIREGISTERMSGEX_FIRSTCOL_INDEX, $GUIREGISTERMSGEX_MESSAGEINDEX, $GUIREGISTERMSGEX_FIRSTROW_INDEX
Global Enum $GUIREGISTERMSGEX_COLUMN_UBOUND, $GUIREGISTERMSGEX_ID, $GUIREGISTERMSGEX_INDEX, $GUIREGISTERMSGEX_RESETCOUNT, $GUIREGISTERMSGEX_ROW_UBOUND, $GUIREGISTERMSGEX_MESSAGESTRING, $GUIREGISTERMSGEX_MAX
; Global Const $GUIREGISTERMSGEX_CALL_ERROR = 0xDEAD
; Global Const $GUIREGISTERMSGEX_CALL_PARAMS = 0xBEEF
#EndRegion Global Variables

Func _GUIRegisterMsgEx($iWM_MESSAGE, $fuDelegate, $bIsUnRegister = False) ; Returns True or False.
	Return __GUIRegisterMsgEx_EVENTPROC($fuDelegate, $iWM_MESSAGE, ($bIsUnRegister ? $GUIREGISTERMSGEX_DELETE : $GUIREGISTERMSGEX_ADD), Null)
EndFunc   ;==>_GUIRegisterMsgEx

Func __GUIRegisterMsgEx_EVENTPROC($hWnd, $iMsg, $wParam, $lParam) ; $hWnd = Func, $iMsg = Windows (WM) message, $wParam = Internal message.
	Local Static $aStorage[$GUIREGISTERMSGEX_FIRSTROW_INDEX][$GUIREGISTERMSGEX_MAX] ; Internal array for storing the WM_MESSAGE and the associated function to call.

	If Not ($aStorage[$GUIREGISTERMSGEX][$GUIREGISTERMSGEX_ID] = $GUIREGISTERMSGEX_GUID) Then ; Initialise the internal array.

		$aStorage[$GUIREGISTERMSGEX][$GUIREGISTERMSGEX_ID] = $GUIREGISTERMSGEX_GUID

		$aStorage[$GUIREGISTERMSGEX][$GUIREGISTERMSGEX_INDEX] = 0

		$aStorage[$GUIREGISTERMSGEX][$GUIREGISTERMSGEX_RESETCOUNT] = 0

		$aStorage[$GUIREGISTERMSGEX][$GUIREGISTERMSGEX_COLUMN_UBOUND] = $GUIREGISTERMSGEX_MAX
		$aStorage[$GUIREGISTERMSGEX][$GUIREGISTERMSGEX_ROW_UBOUND] = $GUIREGISTERMSGEX_FIRSTROW_INDEX

		$aStorage[$GUIREGISTERMSGEX][$GUIREGISTERMSGEX_MESSAGESTRING] = '|'
	EndIf

	Local $iIndex = Int(StringRegExp($aStorage[$GUIREGISTERMSGEX][$GUIREGISTERMSGEX_MESSAGESTRING] & '|' & Int($iMsg) & '*-1|', '\|' & Int($iMsg) & '\*(\-?\d+)\|', $STR_REGEXPARRAYGLOBALMATCH)[0])
	Switch $wParam
		Case $GUIREGISTERMSGEX_ADD
			If $iIndex < 0 Then
				$iIndex = $aStorage[$GUIREGISTERMSGEX][$GUIREGISTERMSGEX_INDEX]
				$aStorage[$GUIREGISTERMSGEX_MESSAGEINDEX][$iIndex] = $GUIREGISTERMSGEX_MESSAGEINDEX
				$aStorage[$GUIREGISTERMSGEX][$GUIREGISTERMSGEX_MESSAGESTRING] &= Int($iMsg) & '*' & $iIndex & '|'
				$aStorage[$GUIREGISTERMSGEX][$GUIREGISTERMSGEX_INDEX] += 1
				GUIRegisterMsg($iMsg, __GUIRegisterMsgEx_EVENTPROC) ; Register the message. Note: Once registered this function will always be associated with that message.
			EndIf

			Local $bIsDelegateExists = False
			For $i = $GUIREGISTERMSGEX_FIRSTROW_INDEX To $aStorage[$GUIREGISTERMSGEX_MESSAGEINDEX][$iIndex] ; Row count.
				If $aStorage[$i][$iIndex] = $hWnd Then
					$bIsDelegateExists = True
					ExitLoop
				EndIf
			Next

			If Not $bIsDelegateExists Then
				$aStorage[$GUIREGISTERMSGEX_MESSAGEINDEX][$iIndex] += 1 ; Increase the count of the message column.

				Local $bIsColAdd = ($iIndex + 1) >= $aStorage[$GUIREGISTERMSGEX][$GUIREGISTERMSGEX_COLUMN_UBOUND], _ ; Re-size the internal array.
						$bIsRowAdd = $aStorage[$GUIREGISTERMSGEX_MESSAGEINDEX][$iIndex] >= $aStorage[$GUIREGISTERMSGEX][$GUIREGISTERMSGEX_ROW_UBOUND]

				If $bIsColAdd Or $bIsRowAdd Then ; Re-size the internal storage if required.
					If $bIsColAdd Then $aStorage[$GUIREGISTERMSGEX][$GUIREGISTERMSGEX_COLUMN_UBOUND] = Ceiling($iIndex * 1.3)
					If $bIsRowAdd Then $aStorage[$GUIREGISTERMSGEX][$GUIREGISTERMSGEX_ROW_UBOUND] = Ceiling($aStorage[$GUIREGISTERMSGEX_MESSAGEINDEX][$iIndex] * 1.3)
					ReDim $aStorage[$aStorage[$GUIREGISTERMSGEX][$GUIREGISTERMSGEX_ROW_UBOUND]][$aStorage[$GUIREGISTERMSGEX][$GUIREGISTERMSGEX_COLUMN_UBOUND]]
				EndIf
				$aStorage[$aStorage[$GUIREGISTERMSGEX_MESSAGEINDEX][$iIndex]][$iIndex] = $hWnd
			EndIf
			Return Not $bIsDelegateExists

		Case $GUIREGISTERMSGEX_DELETE
			Local $bIsDeleted = False
			If $iIndex >= 0 Then
				For $i = $GUIREGISTERMSGEX_FIRSTROW_INDEX To $aStorage[$GUIREGISTERMSGEX_MESSAGEINDEX][$iIndex] ; Row count.
					If $aStorage[$i][$iIndex] = $hWnd Then
						$aStorage[$GUIREGISTERMSGEX][$GUIREGISTERMSGEX_RESETCOUNT] += 1
						$aStorage[$i][$iIndex] = Null
						$bIsDeleted = True
					EndIf
				Next
				If $bIsDeleted And $aStorage[$GUIREGISTERMSGEX][$GUIREGISTERMSGEX_RESETCOUNT] >= 0 Then ; Tidy the internal array by removing empty values.
					Local $iCurrentIndex = $GUIREGISTERMSGEX_FIRSTROW_INDEX
					For $i = $GUIREGISTERMSGEX_FIRSTCOL_INDEX To $aStorage[$GUIREGISTERMSGEX][$GUIREGISTERMSGEX_INDEX] - 1 ; Column count.
						For $j = $GUIREGISTERMSGEX_FIRSTROW_INDEX To $aStorage[$GUIREGISTERMSGEX_MESSAGEINDEX][$i]
							If $aStorage[$j][$i] = Null Then
								$aStorage[$GUIREGISTERMSGEX_MESSAGEINDEX][$i] -= 1
								ContinueLoop
							EndIf
							$aStorage[$iCurrentIndex][$i] = $aStorage[$j][$i]
							$iCurrentIndex += 1
						Next
						$iCurrentIndex = $GUIREGISTERMSGEX_FIRSTROW_INDEX
					Next
				EndIf
			EndIf
			Return $bIsDeleted

		Case Else ; WM_MESSAGE.
			If $iIndex >= 0 And $aStorage[$GUIREGISTERMSGEX_MESSAGEINDEX][$iIndex] >= $GUIREGISTERMSGEX_FIRSTROW_INDEX Then
				For $i = $GUIREGISTERMSGEX_FIRSTROW_INDEX To $aStorage[$GUIREGISTERMSGEX_MESSAGEINDEX][$iIndex] ; Row count.
					If IsFunc($aStorage[$i][$iIndex]) Then
						$aStorage[$i][$iIndex]($hWnd, $iMsg, $wParam, $lParam) ; Execute the registered functions.
					EndIf
					#cs
						; This is not required as all functions that will be registered will contain 4 parameters max.
						If IsFunc($aStorage[$i][$iIndex]) Then
						Call($aStorage[$i][$iIndex], $hWnd, $iMsg, $wParam, $lParam)
						If (@error = $GUIREGISTERMSGEX_CALL_ERROR And @extended = $GUIREGISTERMSGEX_CALL_PARAMS) Then
						Call($aStorage[$i][$iIndex], $hWnd, $iMsg, $wParam)
						If (@error = $GUIREGISTERMSGEX_CALL_ERROR And @extended = $GUIREGISTERMSGEX_CALL_PARAMS) Then
						Call($aStorage[$i][$iIndex], $hWnd, $iMsg)
						If (@error = $GUIREGISTERMSGEX_CALL_ERROR And @extended = $GUIREGISTERMSGEX_CALL_PARAMS) Then
						Call($aStorage[$i][$iIndex], $hWnd)
						If (@error = $GUIREGISTERMSGEX_CALL_ERROR And @extended = $GUIREGISTERMSGEX_CALL_PARAMS) Then
						Call($aStorage[$i][$iIndex])
						EndIf
						EndIf
						EndIf
						EndIf
						EndIf
					#ce
				Next
			EndIf

	EndSwitch

	Return $GUI_RUNDEFMSG
EndFunc   ;==>__GUIRegisterMsgEx_EVENTPROC
